import json
import os
import shutil
from collections import OrderedDict

import regex as re
from ruamel.yaml import RoundTripLoader

from dateparser_scripts.order_languages import avoid_languages
from dateparser_scripts.utils import combine_dicts

cldr_date_directory = '../dateparser_data/cldr_language_data/date_translation_data/'
supplementary_directory = '../dateparser_data/supplementary_language_data/'
supplementary_date_directory = '../dateparser_data/supplementary_language_data/date_translation_data/'
translation_data_directory = '../dateparser/data/'
date_translation_directory = '../dateparser/data/date_translation_data/'

os.chdir(os.path.dirname(os.path.abspath(__file__)))

cldr_languages = list(set(map(lambda x: x[:-5], os.listdir(cldr_date_directory))) - avoid_languages)
supplementary_languages = [x[:-5] for x in os.listdir(supplementary_date_directory)]
all_languages = set(cldr_languages).union(set(supplementary_languages))

RELATIVE_PATTERN = re.compile(r'\{0\}')


def _modify_relative_data(relative_data):
    modified_relative_data = OrderedDict()
    for key, value in relative_data.items():
        for i, string in enumerate(value):
            string = RELATIVE_PATTERN.sub(r'(\\d+[.,]?\\d*)', string)
            value[i] = string
        modified_relative_data[key] = value
    return modified_relative_data


def _modify_data(language_data):
    relative_data = language_data.get("relative-type-regex", {})
    relative_data = _modify_relative_data(relative_data)
    locale_specific_data = language_data.get("locale_specific", {})
    for _, info in locale_specific_data.items():
        locale_relative_data = info.get("relative-type-regex", {})
        locale_relative_data = _modify_relative_data(locale_relative_data)


def _get_complete_date_translation_data(language):
    cldr_data = {}
    supplementary_data = {}
    if language in cldr_languages:
        with open(cldr_date_directory + language + '.json') as f:
            cldr_data = json.load(f, object_pairs_hook=OrderedDict)
    if language in supplementary_languages:
        with open(supplementary_date_directory + language + '.yaml') as g:
            supplementary_data = OrderedDict(RoundTripLoader(g).get_data())
    complete_data = combine_dicts(cldr_data, supplementary_data)
    if 'name' not in complete_data:
        complete_data['name'] = language
    return complete_data


def _write_file(filename, text, mode, in_memory, in_memory_result):
    if in_memory:
        in_memory_result[filename] = text
    else:
        with open(filename, mode) as out:
            out.write(text)


def write_complete_data(in_memory=False):
    """
    This function is responsible of generating the needed py files from the
    CLDR files (JSON format) and supplementary language data (YAML format).

    Use it with in_memory=True to avoid writing real files and getting a
    dictionary containing the file names and their content (used when testing).
    """
    in_memory_result = {}

    if not in_memory:
        if not os.path.isdir(translation_data_directory):
            os.mkdir(translation_data_directory)
        if os.path.isdir(date_translation_directory):
            shutil.rmtree(date_translation_directory)
        os.mkdir(date_translation_directory)

    with open(supplementary_directory + 'base_data.yaml') as f:
        base_data = RoundTripLoader(f).get_data()

    for language in all_languages:
        date_translation_data = _get_complete_date_translation_data(language)
        date_translation_data = combine_dicts(date_translation_data, base_data)
        _modify_data(date_translation_data)
        translation_data = json.dumps(date_translation_data, indent=4, separators=(',', ': '),
                                      ensure_ascii=False)
        out_text = ('info = ' + translation_data + '\n').encode('utf-8')
        _write_file(date_translation_directory + language + '.py', out_text, 'wb', in_memory, in_memory_result)

    init_text = (
        "from dateparser.data import date_translation_data\n"
        "from .languages_info import language_order, language_locale_dict\n"
    )

    _write_file(translation_data_directory + '__init__.py', init_text, 'w', False, in_memory_result)
    _write_file(date_translation_directory + '__init__.py', '', 'w', False, in_memory_result)

    return in_memory_result


if __name__ == '__main__':
    write_complete_data()



info = {
    "name": "fi",
    "date_order": "DMY",
    "january": [
        "tammi",
        "tammik",
        "tammikuu",
        "tammikuuta",
        "tammikuussa"
    ],
    "february": [
        "helmi",
        "helmik",
        "helmikuu",
        "helmikuuta",
        "helmikuussa"
    ],
    "march": [
        "maalis",
        "maalisk",
        "maaliskuu",
        "maaliskuuta",
        "maaliskuussa"
    ],
    "april": [
        "huhti",
        "huhtik",
        "huhtikuu",
        "huhtikuuta",
        "huhtikuussa"
    ],
    "may": [
        "touko",
        "toukok",
        "toukokuu",
        "toukokuuta",
        "toukokuussa"
    ],
    "june": [
        "kesä",
        "kesäk",
        "kesäkuu",
        "kesäkuuta",
        "kesäkuussa"
    ],
    "july": [
        "heinä",
        "heinäk",
        "heinäkuu",
        "heinäkuuta",
        "heinäkuussa"
    ],
    "august": [
        "elo",
        "elok",
        "elokuu",
        "elokuuta",
        "elokuussa"
    ],
    "september": [
        "syys",
        "syysk",
        "syyskuu",
        "syyskuuta",
        "Syyskuussa"
    ],
    "october": [
        "loka",
        "lokak",
        "lokakuu",
        "lokakuuta",
        "Lokakuussa"
    ],
    "november": [
        "marras",
        "marrask",
        "marraskuu",
        "marraskuuta",
        "Marraskuussa"
    ],
    "december": [
        "joulu",
        "jouluk",
        "joulukuu",
        "joulukuuta",
        "Joulukuussa"
    ],
    "monday": [
        "ma",
        "maanantai",
        "maanantaina"
    ],
    "tuesday": [
        "ti",
        "tiistai",
        "tiistaina"
    ],
    "wednesday": [
        "ke",
        "keskiviikko",
        "keskiviikkona"
    ],
    "thursday": [
        "to",
        "torstai",
        "torstaina"
    ],
    "friday": [
        "pe",
        "perjantai",
        "perjantaina"
    ],
    "saturday": [
        "la",
        "lauantai",
        "lauantaina"
    ],
    "sunday": [
        "su",
        "sunnuntai",
        "sunnuntaina"
    ],
    "am": [
        "ap"
    ],
    "pm": [
        "ip"
    ],
    "year": [
        "v",
        "vuosi",
        "vuotta",
        "vv",
        "vuonna",
        "vuoden"
    ],
    "month": [
        "kk",
        "kuukausi",
        "kuukautta",
        "kuukauden"
    ],
    "week": [
        "viikko",
        "vk",
        "viikkoa",
        "vko",
        "viikon"
    ],
    "day": [
        "pv",
        "päivä",
        "päivää",
        "p",
        "pvä",
        "pvää",
        "päivän"
    ],
    "hour": [
        "t",
        "tunti",
        "tuntia",
        "tunnin"
    ],
    "minute": [
        "min",
        "minuutti",
        "minuuttia",
        "minuutin"
    ],
    "second": [
        "s",
        "sekunti",
        "sekuntia",
        "sekuntti",
        "sekunttia",
        "sekuntin",
        "sekunnin"
    ],
    "relative-type": {
        "0 day ago": [
            "tänään"
        ],
        "0 hour ago": [
            "tunnin sisällä",
            "tämän tunnin aikana"
        ],
        "0 minute ago": [
            "minuutin sisällä",
            "tämän minuutin aikana"
        ],
        "0 month ago": [
            "tässä kk",
            "tässä kuussa"
        ],
        "0 second ago": [
            "nyt"
        ],
        "0 week ago": [
            "tällä viikolla",
            "tällä vk"
        ],
        "0 year ago": [
            "tänä v",
            "tänä vuonna"
        ],
        "1 day ago": [
            "eilen"
        ],
        "1 month ago": [
            "viime kk",
            "viime kuussa"
        ],
        "1 week ago": [
            "viime viikolla",
            "viime vk"
        ],
        "1 year ago": [
            "viime v",
            "viime vuonna"
        ],
        "in 1 day": [
            "huom",
            "huomenna"
        ],
        "in 1 month": [
            "ensi kk",
            "ensi kuussa"
        ],
        "in 1 week": [
            "ensi viikolla",
            "ensi vk"
        ],
        "in 1 year": [
            "ensi v",
            "ensi vuonna"
        ],
        "2 year ago": [
            "toissa vuonna"
        ],
        "2 month ago": [
            "toissa kuussa"
        ],
        "2 week ago": [
            "toissa viikolla"
        ],
        "2 day ago": [
            "toissa päivänä"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) pv sitten",
            "(\\d+[.,]?\\d*) päivä sitten",
            "(\\d+[.,]?\\d*) päivää sitten"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) t sitten",
            "(\\d+[.,]?\\d*) tunti sitten",
            "(\\d+[.,]?\\d*) tuntia sitten"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) min sitten",
            "(\\d+[.,]?\\d*) minuutti sitten",
            "(\\d+[.,]?\\d*) minuuttia sitten"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) kk sitten",
            "(\\d+[.,]?\\d*) kuukausi sitten",
            "(\\d+[.,]?\\d*) kuukautta sitten"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) s sitten",
            "(\\d+[.,]?\\d*) sekunti sitten",
            "(\\d+[.,]?\\d*) sekuntia sitten"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) viikko sitten",
            "(\\d+[.,]?\\d*) viikkoa sitten",
            "(\\d+[.,]?\\d*) vk sitten"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) v sitten",
            "(\\d+[.,]?\\d*) vuosi sitten",
            "(\\d+[.,]?\\d*) vuotta sitten"
        ],
        "in \\1 day": [
            "(\\d+[.,]?\\d*) pv päästä",
            "(\\d+[.,]?\\d*) päivän päästä"
        ],
        "in \\1 hour": [
            "(\\d+[.,]?\\d*) t päästä",
            "(\\d+[.,]?\\d*) tunnin päästä"
        ],
        "in \\1 minute": [
            "(\\d+[.,]?\\d*) min päästä",
            "(\\d+[.,]?\\d*) minuutin päästä"
        ],
        "in \\1 month": [
            "(\\d+[.,]?\\d*) kk päästä",
            "(\\d+[.,]?\\d*) kuukauden päästä"
        ],
        "in \\1 second": [
            "(\\d+[.,]?\\d*) s päästä",
            "(\\d+[.,]?\\d*) sekunnin päästä"
        ],
        "in \\1 week": [
            "(\\d+[.,]?\\d*) viikon päästä",
            "(\\d+[.,]?\\d*) vk päästä"
        ],
        "in \\1 year": [
            "(\\d+[.,]?\\d*) v päästä",
            "(\\d+[.,]?\\d*) vuoden päästä"
        ]
    },
    "locale_specific": {},
    "skip": [
        ":n",
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ],
    "sentence_splitter_group": 1,
    "ago": [
        "sitten"
    ],
    "in": [
        "kuluttua",
        "päästä"
    ],
    "simplifications": [
        {
            "(\\d+[.,]?\\d*) (sekunnin|sekuntin|minuutin|tunnin|päivän|viikon|kuukauden|vuoden) (päästä|kuluttua)": "\\3 \\1 \\2"
        }
    ]
}



info = {
    "name": "xog",
    "date_order": "DMY",
    "january": [
        "jan",
        "janwaliyo"
    ],
    "february": [
        "feb",
        "febwaliyo"
    ],
    "march": [
        "mar",
        "marisi"
    ],
    "april": [
        "apu",
        "apuli"
    ],
    "may": [
        "maa",
        "maayi"
    ],
    "june": [
        "juu",
        "juuni"
    ],
    "july": [
        "jul",
        "julaayi"
    ],
    "august": [
        "agu",
        "agusito"
    ],
    "september": [
        "seb",
        "sebuttemba"
    ],
    "october": [
        "oki",
        "okitobba"
    ],
    "november": [
        "nov",
        "novemba"
    ],
    "december": [
        "des",
        "desemba"
    ],
    "monday": [
        "bala",
        "balaza"
    ],
    "tuesday": [
        "kubi",
        "owokubili"
    ],
    "wednesday": [
        "kusa",
        "owokusatu"
    ],
    "thursday": [
        "kuna",
        "olokuna"
    ],
    "friday": [
        "kuta",
        "olokutaanu"
    ],
    "saturday": [
        "muka",
        "olomukaaga"
    ],
    "sunday": [
        "sabi",
        "sabiiti"
    ],
    "am": [
        "munkyo"
    ],
    "pm": [
        "eigulo"
    ],
    "year": [
        "omwaka"
    ],
    "month": [
        "omwezi"
    ],
    "week": [
        "esabiiti"
    ],
    "day": [
        "olunaku"
    ],
    "hour": [
        "essawa"
    ],
    "minute": [
        "edakiika"
    ],
    "second": [
        "obutikitiki"
    ],
    "relative-type": {
        "0 day ago": [
            "olwaleelo (leelo)"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "edho"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "enkyo"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "os",
    "date_order": "DMY",
    "january": [
        "янв",
        "январы",
        "январь"
    ],
    "february": [
        "фев",
        "февр",
        "февралы",
        "февраль"
    ],
    "march": [
        "мар",
        "март",
        "мартъи",
        "мартъийы"
    ],
    "april": [
        "апр",
        "апрелы",
        "апрель"
    ],
    "may": [
        "май",
        "майы"
    ],
    "june": [
        "июны",
        "июнь"
    ],
    "july": [
        "июлы",
        "июль"
    ],
    "august": [
        "авг",
        "август",
        "августы"
    ],
    "september": [
        "сен",
        "сент",
        "сентябры",
        "сентябрь"
    ],
    "october": [
        "окт",
        "октябры",
        "октябрь"
    ],
    "november": [
        "ноя",
        "нояб",
        "ноябры",
        "ноябрь"
    ],
    "december": [
        "дек",
        "декабры",
        "декабрь"
    ],
    "monday": [
        "крс",
        "къуырисӕр"
    ],
    "tuesday": [
        "дцг",
        "дыццӕг"
    ],
    "wednesday": [
        "ӕрт",
        "ӕртыццӕг"
    ],
    "thursday": [
        "цпр",
        "цыппӕрӕм"
    ],
    "friday": [
        "майрӕмбон",
        "мрб"
    ],
    "saturday": [
        "сабат",
        "сбт"
    ],
    "sunday": [
        "хуыцаубон",
        "хцб"
    ],
    "am": [
        "am",
        "ӕмбисбоны размӕ"
    ],
    "pm": [
        "pm",
        "ӕмбисбоны фӕстӕ"
    ],
    "year": [
        "аз"
    ],
    "month": [
        "мӕй"
    ],
    "week": [
        "къуыри"
    ],
    "day": [
        "бон"
    ],
    "hour": [
        "сахат"
    ],
    "minute": [
        "минут"
    ],
    "second": [
        "секунд"
    ],
    "relative-type": {
        "0 day ago": [
            "абон"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "знон"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "сом"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) бон раздӕр",
            "(\\d+[.,]?\\d*) боны размӕ"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) сахаты размӕ"
        ],
        "in \\1 day": [
            "(\\d+[.,]?\\d*) боны фӕстӕ"
        ],
        "in \\1 hour": [
            "(\\d+[.,]?\\d*) сахаты фӕстӕ"
        ]
    },
    "locale_specific": {
        "os-RU": {
            "name": "os-RU"
        }
    },
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



import regex as re
from datetime import datetime
from datetime import time
from tzlocal import get_localzone

from dateutil.relativedelta import relativedelta

from dateparser.utils import apply_timezone, localize_timezone, strip_braces
from .parser import time_parser
from .timezone_parser import pop_tz_offset_from_string


_UNITS = r'decade|year|month|week|day|hour|minute|second'
PATTERN = re.compile(r'(\d+[.,]?\d*)\s*(%s)\b' % _UNITS, re.I | re.S | re.U)


class FreshnessDateDataParser:
    """ Parses date string like "1 year, 2 months ago" and "3 hours, 50 minutes ago" """

    def _are_all_words_units(self, date_string):
        skip = [_UNITS,
                r'ago|in|\d+',
                r':|[ap]m']

        date_string = re.sub(r'\s+', ' ', date_string.strip())

        words = [x for x in re.split(r'\W', date_string) if x]
        words = [x for x in words if not re.match(r'%s' % '|'.join(skip), x)]
        return not words

    def _parse_time(self, date_string, settings):
        """Attempts to parse time part of date strings like '1 day ago, 2 PM' """
        date_string = PATTERN.sub('', date_string)
        date_string = re.sub(r'\b(?:ago|in)\b', '', date_string)
        try:
            return time_parser(date_string)
        except:
            pass

    def get_local_tz(self):
        return get_localzone()

    def parse(self, date_string, settings):
        date_string = strip_braces(date_string)
        date_string, ptz = pop_tz_offset_from_string(date_string)
        _time = self._parse_time(date_string, settings)

        _settings_tz = settings.TIMEZONE.lower()

        def apply_time(dateobj, timeobj):
            if not isinstance(_time, time):
                return dateobj

            return dateobj.replace(
                hour=timeobj.hour, minute=timeobj.minute,
                second=timeobj.second, microsecond=timeobj.microsecond
            )

        if settings.RELATIVE_BASE:
            now = settings.RELATIVE_BASE

            if 'local' not in _settings_tz:
                now = localize_timezone(now, settings.TIMEZONE)

            if ptz:
                if now.tzinfo:
                    now = now.astimezone(ptz)
                else:
                    if hasattr(ptz, 'localize'):
                        now = ptz.localize(now)
                    else:
                        now = now.replace(tzinfo=ptz)

            if not now.tzinfo:
                if hasattr(self.get_local_tz(), 'localize'):
                    now = self.get_local_tz().localize(now)
                else:
                    now = now.replace(tzinfo=self.get_local_tz())

        elif ptz:
            localized_now = datetime.now(ptz)

            if 'local' in _settings_tz:
                now = localized_now
            else:
                now = apply_timezone(localized_now, settings.TIMEZONE)

        else:
            if 'local' not in _settings_tz:
                utc_dt = datetime.utcnow()
                now = apply_timezone(utc_dt, settings.TIMEZONE)
            else:
                now = datetime.now(self.get_local_tz())

        date, period = self._parse_date(date_string, now, settings.PREFER_DATES_FROM)

        if date:
            old_date = date
            date = apply_time(date, _time)
            if settings.RETURN_TIME_AS_PERIOD and old_date != date:
                period = 'time'

            if settings.TO_TIMEZONE:
                date = apply_timezone(date, settings.TO_TIMEZONE)

            if (
                not settings.RETURN_AS_TIMEZONE_AWARE
                or (settings.RETURN_AS_TIMEZONE_AWARE
                    and 'default' == settings.RETURN_AS_TIMEZONE_AWARE and not ptz)
            ):
                date = date.replace(tzinfo=None)

        return date, period

    def _parse_date(self, date_string, now, prefer_dates_from):
        if not self._are_all_words_units(date_string):
            return None, None

        kwargs = self.get_kwargs(date_string)
        if not kwargs:
            return None, None
        period = 'day'
        if 'days' not in kwargs:
            for k in ['weeks', 'months', 'years']:
                if k in kwargs:
                    period = k[:-1]
                    break
        td = relativedelta(**kwargs)

        if (
            re.search(r'\bin\b', date_string)
            or re.search(r'\bfuture\b', prefer_dates_from)
            and not re.search(r'\bago\b', date_string)
        ):
            date = now + td
        else:
            date = now - td
        return date, period

    def get_kwargs(self, date_string):
        m = PATTERN.findall(date_string)
        if not m:
            return {}

        kwargs = {}
        for num, unit in m:
            kwargs[unit + 's'] = float(num.replace(",", "."))
        if 'decades' in kwargs:
            kwargs['years'] = 10 * kwargs['decades'] + kwargs.get('years', 0)
            del kwargs['decades']
        return kwargs

    def get_date_data(self, date_string, settings=None):
        from dateparser.date import DateData

        date, period = self.parse(date_string, settings)
        return DateData(date_obj=date, period=period)


freshness_date_parser = FreshnessDateDataParser()



info = {
    "name": "naq",
    "date_order": "DMY",
    "january": [
        "jan",
        "ǃkhanni"
    ],
    "february": [
        "feb",
        "ǃkhanǀgôab"
    ],
    "march": [
        "mar",
        "ǀkhuuǁkhâb"
    ],
    "april": [
        "apr",
        "ǃhôaǂkhaib"
    ],
    "may": [
        "may",
        "ǃkhaitsâb"
    ],
    "june": [
        "gamaǀaeb",
        "jun"
    ],
    "july": [
        "jul",
        "ǂkhoesaob"
    ],
    "august": [
        "aoǁkhuumûǁkhâb",
        "aug"
    ],
    "september": [
        "sep",
        "taraǀkhuumûǁkhâb"
    ],
    "october": [
        "oct",
        "ǂnûǁnâiseb"
    ],
    "november": [
        "nov",
        "ǀhooǂgaeb"
    ],
    "december": [
        "dec",
        "hôasoreǁkhâb"
    ],
    "monday": [
        "ma",
        "mantaxtsees"
    ],
    "tuesday": [
        "de",
        "denstaxtsees"
    ],
    "wednesday": [
        "wu",
        "wunstaxtsees"
    ],
    "thursday": [
        "do",
        "dondertaxtsees"
    ],
    "friday": [
        "fr",
        "fraitaxtsees"
    ],
    "saturday": [
        "sat",
        "satertaxtsees"
    ],
    "sunday": [
        "son",
        "sontaxtsees"
    ],
    "am": [
        "ǁgoagas"
    ],
    "pm": [
        "ǃuias"
    ],
    "year": [
        "kurib"
    ],
    "month": [
        "ǁkhâb"
    ],
    "week": [
        "wekheb"
    ],
    "day": [
        "tsees"
    ],
    "hour": [
        "iiri"
    ],
    "minute": [
        "haib"
    ],
    "second": [
        "ǀgâub"
    ],
    "relative-type": {
        "0 day ago": [
            "neetsee"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "yesterday"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "tomorrow"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "seh",
    "date_order": "DMY",
    "january": [
        "jan",
        "janeiro"
    ],
    "february": [
        "fev",
        "fevreiro"
    ],
    "march": [
        "mar",
        "marco"
    ],
    "april": [
        "abr",
        "abril"
    ],
    "may": [
        "mai",
        "maio"
    ],
    "june": [
        "jun",
        "junho"
    ],
    "july": [
        "jul",
        "julho"
    ],
    "august": [
        "aug",
        "augusto"
    ],
    "september": [
        "set",
        "setembro"
    ],
    "october": [
        "otu",
        "otubro"
    ],
    "november": [
        "nov",
        "novembro"
    ],
    "december": [
        "dec",
        "decembro"
    ],
    "monday": [
        "chiposi",
        "pos"
    ],
    "tuesday": [
        "chipiri",
        "pir"
    ],
    "wednesday": [
        "chitatu",
        "tat"
    ],
    "thursday": [
        "chinai",
        "nai"
    ],
    "friday": [
        "chishanu",
        "sha"
    ],
    "saturday": [
        "sab",
        "sabudu"
    ],
    "sunday": [
        "dim",
        "dimingu"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "chaka"
    ],
    "month": [
        "mwezi"
    ],
    "week": [
        "week"
    ],
    "day": [
        "ntsiku"
    ],
    "hour": [
        "hora"
    ],
    "minute": [
        "minuto"
    ],
    "second": [
        "segundo"
    ],
    "relative-type": {
        "0 day ago": [
            "lero"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "zuro"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "manguana"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "kam",
    "date_order": "DMY",
    "january": [
        "mbe",
        "mwai wa mbee"
    ],
    "february": [
        "kel",
        "mwai wa kelĩ"
    ],
    "march": [
        "ktũ",
        "mwai wa katatũ"
    ],
    "april": [
        "kan",
        "mwai wa kana"
    ],
    "may": [
        "ktn",
        "mwai wa katano"
    ],
    "june": [
        "mwai wa thanthatũ",
        "tha"
    ],
    "july": [
        "moo",
        "mwai wa muonza"
    ],
    "august": [
        "mwai wa nyaanya",
        "nya"
    ],
    "september": [
        "knd",
        "mwai wa kenda"
    ],
    "october": [
        "mwai wa ĩkumi",
        "ĩku"
    ],
    "november": [
        "mwai wa ĩkumi na ĩmwe",
        "ĩkm"
    ],
    "december": [
        "mwai wa ĩkumi na ilĩ",
        "ĩkl"
    ],
    "monday": [
        "wa kwambĩlĩlya",
        "wkw"
    ],
    "tuesday": [
        "wa kelĩ",
        "wkl"
    ],
    "wednesday": [
        "wa katatũ",
        "wtũ"
    ],
    "thursday": [
        "wa kana",
        "wkn"
    ],
    "friday": [
        "wa katano",
        "wtn"
    ],
    "saturday": [
        "wa thanthatũ",
        "wth"
    ],
    "sunday": [
        "wa kyumwa",
        "wky"
    ],
    "am": [
        "ĩyakwakya"
    ],
    "pm": [
        "ĩyawĩoo"
    ],
    "year": [
        "mwaka"
    ],
    "month": [
        "mwai"
    ],
    "week": [
        "kyumwa"
    ],
    "day": [
        "mũthenya"
    ],
    "hour": [
        "saa"
    ],
    "minute": [
        "ndatĩka"
    ],
    "second": [
        "sekondi"
    ],
    "relative-type": {
        "0 day ago": [
            "ũmũnthĩ"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "ĩyoo"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "ũnĩ"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "bez",
    "date_order": "DMY",
    "january": [
        "hut",
        "pa mwedzi gwa hutala"
    ],
    "february": [
        "pa mwedzi gwa wuvili",
        "vil"
    ],
    "march": [
        "dat",
        "pa mwedzi gwa wudatu"
    ],
    "april": [
        "pa mwedzi gwa wutai",
        "tai"
    ],
    "may": [
        "han",
        "pa mwedzi gwa wuhanu"
    ],
    "june": [
        "pa mwedzi gwa sita",
        "sit"
    ],
    "july": [
        "pa mwedzi gwa saba",
        "sab"
    ],
    "august": [
        "nan",
        "pa mwedzi gwa nane"
    ],
    "september": [
        "pa mwedzi gwa tisa",
        "tis"
    ],
    "october": [
        "kum",
        "pa mwedzi gwa kumi"
    ],
    "november": [
        "kmj",
        "pa mwedzi gwa kumi na moja"
    ],
    "december": [
        "kmb",
        "pa mwedzi gwa kumi na mbili"
    ],
    "monday": [
        "pa shahuviluha",
        "vil"
    ],
    "tuesday": [
        "hiv",
        "pa hivili"
    ],
    "wednesday": [
        "hid",
        "pa hidatu"
    ],
    "thursday": [
        "hit",
        "pa hitayi"
    ],
    "friday": [
        "hih",
        "pa hihanu"
    ],
    "saturday": [
        "lem",
        "pa shahulembela"
    ],
    "sunday": [
        "mul",
        "pa mulungu"
    ],
    "am": [
        "pamilau"
    ],
    "pm": [
        "pamunyi"
    ],
    "year": [
        "mwaha"
    ],
    "month": [
        "mwedzi"
    ],
    "week": [
        "mlungu gumamfu"
    ],
    "day": [
        "sihu"
    ],
    "hour": [
        "saa"
    ],
    "minute": [
        "dakika"
    ],
    "second": [
        "sekunde"
    ],
    "relative-type": {
        "0 day ago": [
            "neng'u ni"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "igolo"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "hilawu"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "uz-Cyrl",
    "date_order": "DMY",
    "january": [
        "янв",
        "январ"
    ],
    "february": [
        "фев",
        "феврал"
    ],
    "march": [
        "мар",
        "март"
    ],
    "april": [
        "апр",
        "апрел"
    ],
    "may": [
        "май"
    ],
    "june": [
        "июн"
    ],
    "july": [
        "июл"
    ],
    "august": [
        "авг",
        "август"
    ],
    "september": [
        "сен",
        "сентябр"
    ],
    "october": [
        "окт",
        "октябр"
    ],
    "november": [
        "ноя",
        "ноябр"
    ],
    "december": [
        "дек",
        "декабр"
    ],
    "monday": [
        "душ",
        "душанба"
    ],
    "tuesday": [
        "сеш",
        "сешанба"
    ],
    "wednesday": [
        "чор",
        "чоршанба"
    ],
    "thursday": [
        "пай",
        "пайшанба"
    ],
    "friday": [
        "жум",
        "жума"
    ],
    "saturday": [
        "шан",
        "шанба"
    ],
    "sunday": [
        "якш",
        "якшанба"
    ],
    "am": [
        "то"
    ],
    "pm": [
        "тк"
    ],
    "year": [
        "йил"
    ],
    "month": [
        "ой"
    ],
    "week": [
        "ҳафта"
    ],
    "day": [
        "кун"
    ],
    "hour": [
        "соат"
    ],
    "minute": [
        "дақиқа"
    ],
    "second": [
        "сония"
    ],
    "relative-type": {
        "0 day ago": [
            "бугун"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "бу ой"
        ],
        "0 second ago": [
            "ҳозир"
        ],
        "0 week ago": [
            "бу ҳафта"
        ],
        "0 year ago": [
            "бу йил"
        ],
        "1 day ago": [
            "кеча"
        ],
        "1 month ago": [
            "ўтган ой"
        ],
        "1 week ago": [
            "ўтган ҳафта"
        ],
        "1 year ago": [
            "ўтган йил"
        ],
        "in 1 day": [
            "эртага"
        ],
        "in 1 month": [
            "кейинги ой"
        ],
        "in 1 week": [
            "кейинги ҳафта"
        ],
        "in 1 year": [
            "кейинги йил"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) кун олдин"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) соат олдин"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) дақиқа олдин"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) ой аввал"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) сония олдин"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) ҳафта олдин"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) йил аввал"
        ],
        "in \\1 day": [
            "(\\d+[.,]?\\d*) кундан сўнг"
        ],
        "in \\1 hour": [
            "(\\d+[.,]?\\d*) соатдан сўнг"
        ],
        "in \\1 minute": [
            "(\\d+[.,]?\\d*) дақиқадан сўнг"
        ],
        "in \\1 month": [
            "(\\d+[.,]?\\d*) ойдан сўнг"
        ],
        "in \\1 second": [
            "(\\d+[.,]?\\d*) сониядан сўнг"
        ],
        "in \\1 week": [
            "(\\d+[.,]?\\d*) ҳафтадан сўнг"
        ],
        "in \\1 year": [
            "(\\d+[.,]?\\d*) йилдан сўнг"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "bas",
    "date_order": "DMY",
    "january": [
        "kɔn",
        "kɔndɔŋ"
    ],
    "february": [
        "mac",
        "màcɛ̂l"
    ],
    "march": [
        "mat",
        "màtùmb"
    ],
    "april": [
        "mto",
        "màtop"
    ],
    "may": [
        "mpu",
        "m̀puyɛ"
    ],
    "june": [
        "hil",
        "hìlòndɛ̀"
    ],
    "july": [
        "nje",
        "njèbà"
    ],
    "august": [
        "hik",
        "hìkaŋ"
    ],
    "september": [
        "dip",
        "dìpɔ̀s"
    ],
    "october": [
        "bio",
        "bìòôm"
    ],
    "november": [
        "may",
        "màyɛsèp"
    ],
    "december": [
        "liɓ",
        "lìbuy li ńyèe"
    ],
    "monday": [
        "nja",
        "ŋgwà njaŋgumba"
    ],
    "tuesday": [
        "uum",
        "ŋgwà ûm"
    ],
    "wednesday": [
        "ŋge",
        "ŋgwà ŋgê"
    ],
    "thursday": [
        "mbɔ",
        "ŋgwà mbɔk"
    ],
    "friday": [
        "kɔɔ",
        "ŋgwà kɔɔ"
    ],
    "saturday": [
        "jon",
        "ŋgwà jôn"
    ],
    "sunday": [
        "nɔy",
        "ŋgwà nɔ̂y"
    ],
    "am": [
        "i bikɛ̂glà"
    ],
    "pm": [
        "i ɓugajɔp"
    ],
    "year": [
        "ŋwìi"
    ],
    "month": [
        "soŋ"
    ],
    "week": [
        "sɔndɛ̂"
    ],
    "day": [
        "kɛl"
    ],
    "hour": [
        "ŋgɛŋ"
    ],
    "minute": [
        "ŋget"
    ],
    "second": [
        "hìŋgeŋget"
    ],
    "relative-type": {
        "0 day ago": [
            "lɛ̀n"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "yààni"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "yàni"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



import sys
import os
from pathlib import Path

DEFAULT_DIR_NAME = 'dateparser_models'
DEFAULT_UNIX_CACHE_DIR = '~/.cache'

DEFAULT_WINDOWS_CACHE_DIR = os.path.join(Path.home(), "AppData", "Roaming")


if sys.platform.startswith('win'):
    # For Windows:
    _cache_dir = DEFAULT_WINDOWS_CACHE_DIR
else:
    # UNIX & OS X:
    _cache_dir = DEFAULT_UNIX_CACHE_DIR

dateparser_model_home = os.path.expanduser(
    os.path.join(_cache_dir, DEFAULT_DIR_NAME)
)


def create_data_model_home():
    os.makedirs(dateparser_model_home, exist_ok=True)


def clear_cache(*args):
    for path in Path(dateparser_model_home).rglob('*.*'):
        os.remove(path)



info = {
    "name": "so",
    "date_order": "DMY",
    "january": [
        "bisha koobaad",
        "kob"
    ],
    "february": [
        "bisha labaad",
        "lab"
    ],
    "march": [
        "bisha saddexaad",
        "sad"
    ],
    "april": [
        "afr",
        "bisha afraad"
    ],
    "may": [
        "bisha shanaad",
        "sha"
    ],
    "june": [
        "bisha lixaad",
        "lix"
    ],
    "july": [
        "bisha todobaad",
        "tod"
    ],
    "august": [
        "bisha sideedaad",
        "sid"
    ],
    "september": [
        "bisha sagaalaad",
        "sag"
    ],
    "october": [
        "bisha tobnaad",
        "tob"
    ],
    "november": [
        "bisha kow iyo tobnaad",
        "kit"
    ],
    "december": [
        "bisha laba iyo tobnaad",
        "lit"
    ],
    "monday": [
        "isn",
        "isniin"
    ],
    "tuesday": [
        "tal",
        "talaado"
    ],
    "wednesday": [
        "arb",
        "arbaco"
    ],
    "thursday": [
        "kha",
        "khamiis"
    ],
    "friday": [
        "jim",
        "jimco"
    ],
    "saturday": [
        "sab",
        "sabti"
    ],
    "sunday": [
        "axad",
        "axd"
    ],
    "am": [
        "sn"
    ],
    "pm": [
        "gn"
    ],
    "year": [
        "year"
    ],
    "month": [
        "month"
    ],
    "week": [
        "week"
    ],
    "day": [
        "day"
    ],
    "hour": [
        "hour"
    ],
    "minute": [
        "minute"
    ],
    "second": [
        "second"
    ],
    "relative-type": {
        "0 day ago": [
            "maanta"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "shalay"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "berri"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {
        "so-DJ": {
            "name": "so-DJ"
        },
        "so-ET": {
            "name": "so-ET"
        },
        "so-KE": {
            "name": "so-KE"
        }
    },
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



print("import: 'dateparser'")
import dateparser

print("import: 'dateparser.calendars'")
import dateparser.calendars

print("import: 'dateparser.languages'")
import dateparser.languages

print("import: 'dateparser.utils'")
import dateparser.utils




info = {
    "name": "lkt",
    "date_order": "YMD",
    "january": [
        "wiótheȟika wí"
    ],
    "february": [
        "thiyóȟeyuŋka wí"
    ],
    "march": [
        "ištáwičhayazaŋ wí"
    ],
    "april": [
        "pȟežítȟo wí"
    ],
    "may": [
        "čhaŋwápetȟo wí"
    ],
    "june": [
        "wípazukȟa-wašté wí"
    ],
    "july": [
        "čhaŋpȟásapa wí"
    ],
    "august": [
        "wasútȟuŋ wí"
    ],
    "september": [
        "čhaŋwápeǧi wí"
    ],
    "october": [
        "čhaŋwápe-kasná wí"
    ],
    "november": [
        "waníyetu wí"
    ],
    "december": [
        "tȟahékapšuŋ wí"
    ],
    "monday": [
        "aŋpétuwaŋži"
    ],
    "tuesday": [
        "aŋpétunuŋpa"
    ],
    "wednesday": [
        "aŋpétuyamni"
    ],
    "thursday": [
        "aŋpétutopa"
    ],
    "friday": [
        "aŋpétuzaptaŋ"
    ],
    "saturday": [
        "owáŋgyužažapi"
    ],
    "sunday": [
        "aŋpétuwakȟaŋ"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "ómakȟa"
    ],
    "month": [
        "wí"
    ],
    "week": [
        "okó"
    ],
    "day": [
        "aŋpétu"
    ],
    "hour": [
        "owápȟe"
    ],
    "minute": [
        "owápȟe oȟ'áŋkȟo"
    ],
    "second": [
        "okpí"
    ],
    "relative-type": {
        "0 day ago": [
            "lé aŋpétu kiŋ"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "lé wí kiŋ"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "lé okó kiŋ"
        ],
        "0 year ago": [
            "lé ómakȟa kiŋ"
        ],
        "1 day ago": [
            "ȟtálehaŋ"
        ],
        "1 month ago": [
            "wí k'uŋ héhaŋ"
        ],
        "1 week ago": [
            "okó k'uŋ héhaŋ"
        ],
        "1 year ago": [
            "ómakȟa k'uŋ héhaŋ"
        ],
        "in 1 day": [
            "híŋhaŋni kiŋháŋ"
        ],
        "in 1 month": [
            "tȟokáta wí kiŋháŋ"
        ],
        "in 1 week": [
            "tȟokáta okó kiŋháŋ"
        ],
        "in 1 year": [
            "tȟokáta ómakȟa kiŋháŋ"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "hékta (\\d+[.,]?\\d*)-čháŋ k'uŋ héhaŋ"
        ],
        "\\1 hour ago": [
            "hékta owápȟe (\\d+[.,]?\\d*) k'uŋ héhaŋ"
        ],
        "\\1 minute ago": [
            "hékta oȟ'áŋkȟo (\\d+[.,]?\\d*) k'uŋ héhaŋ"
        ],
        "\\1 month ago": [
            "hékta wíyawapi (\\d+[.,]?\\d*) k'uŋ héhaŋ"
        ],
        "\\1 second ago": [
            "hékta okpí (\\d+[.,]?\\d*) k'uŋ héhaŋ"
        ],
        "\\1 week ago": [
            "hékta okó (\\d+[.,]?\\d*) k'uŋ héhaŋ"
        ],
        "\\1 year ago": [
            "hékta ómakȟa (\\d+[.,]?\\d*) k'uŋ héhaŋ"
        ],
        "in \\1 day": [
            "letáŋhaŋ (\\d+[.,]?\\d*)-čháŋ kiŋháŋ"
        ],
        "in \\1 hour": [
            "letáŋhaŋ owápȟe (\\d+[.,]?\\d*) kiŋháŋ"
        ],
        "in \\1 minute": [
            "letáŋhaŋ oȟ'áŋkȟo (\\d+[.,]?\\d*) kiŋháŋ"
        ],
        "in \\1 month": [
            "letáŋhaŋ wíyawapi (\\d+[.,]?\\d*) kiŋháŋ"
        ],
        "in \\1 second": [
            "letáŋhaŋ okpí (\\d+[.,]?\\d*) kiŋháŋ"
        ],
        "in \\1 week": [
            "letáŋhaŋ okó (\\d+[.,]?\\d*) kiŋháŋ"
        ],
        "in \\1 year": [
            "letáŋhaŋ ómakȟa (\\d+[.,]?\\d*) kiŋháŋ"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



from itertools import chain, zip_longest
from operator import methodcaller
import regex as re

from dateparser.utils import normalize_unicode

PARSER_HARDCODED_TOKENS = [":", ".", " ", "-", "/"]
PARSER_KNOWN_TOKENS = ["am", "pm", "UTC", "GMT", "Z"]
ALWAYS_KEEP_TOKENS = ["+"] + PARSER_HARDCODED_TOKENS
KNOWN_WORD_TOKENS = ['monday', 'tuesday', 'wednesday', 'thursday', 'friday',
                     'saturday', 'sunday', 'january', 'february', 'march',
                     'april', 'may', 'june', 'july', 'august', 'september',
                     'october', 'november', 'december', 'decade', 'year',
                     'month', 'week', 'day', 'hour', 'minute', 'second', 'ago',
                     'in', 'am', 'pm']

PARENTHESES_PATTERN = re.compile(r'[\(\)]')
NUMERAL_PATTERN = re.compile(r'(\d+)')
KEEP_TOKEN_PATTERN = re.compile(r"^.*[^\W_].*$", flags=re.U)


class UnknownTokenError(Exception):
    pass


class Dictionary:
    """
    Class that modifies and stores translations and handles splitting of date string.

    :param locale_info:
        Locale info (translation data) of the locale.
    :type language_info: dict

    :param settings:
        Configure customized behavior using settings defined in :mod:`dateparser.conf.Settings`.
    :type settings: dict

    :return: a Dictionary instance.
    """

    _split_regex_cache = {}
    _sorted_words_cache = {}
    _split_relative_regex_cache = {}
    _sorted_relative_strings_cache = {}
    _match_relative_regex_cache = {}

    def __init__(self, locale_info, settings=None):
        dictionary = {}
        self._settings = settings
        self.info = locale_info

        if 'skip' in locale_info:
            skip = map(methodcaller('lower'), locale_info['skip'])
            dictionary.update(zip_longest(skip, [], fillvalue=None))
        if 'pertain' in locale_info:
            pertain = map(methodcaller('lower'), locale_info['pertain'])
            dictionary.update(zip_longest(pertain, [], fillvalue=None))
        for word in KNOWN_WORD_TOKENS:
            if word in locale_info:
                translations = map(methodcaller('lower'), locale_info[word])
                dictionary.update(zip_longest(translations, [], fillvalue=word))
        dictionary.update(zip_longest(ALWAYS_KEEP_TOKENS, ALWAYS_KEEP_TOKENS))
        dictionary.update(zip_longest(map(methodcaller('lower'),
                                          PARSER_KNOWN_TOKENS),
                                      PARSER_KNOWN_TOKENS))

        relative_type = locale_info.get('relative-type', {})
        for key, value in relative_type.items():
            relative_translations = map(methodcaller('lower'), value)
            dictionary.update(zip_longest(relative_translations, [], fillvalue=key))

        self._dictionary = dictionary

        no_word_spacing = locale_info.get('no_word_spacing', 'False')
        self._no_word_spacing = bool(eval(no_word_spacing))

        relative_type_regex = locale_info.get("relative-type-regex", {})
        self._relative_strings = list(chain.from_iterable(relative_type_regex.values()))

    def __contains__(self, key):
        if key in self._settings.SKIP_TOKENS:
            return True
        return self._dictionary.__contains__(key)

    def __getitem__(self, key):
        if key in self._settings.SKIP_TOKENS:
            return None
        return self._dictionary.__getitem__(key)

    def __iter__(self):
        return chain(self._settings.SKIP_TOKENS, iter(self._dictionary))

    def are_tokens_valid(self, tokens):
        """
        Check if tokens are valid tokens for the locale.

        :param tokens:
            a list of string tokens.
        :type tokens: list

        :return: True if tokens are valid, False otherwise.
        """
        has_only_keep_tokens = not set(tokens) - set(ALWAYS_KEEP_TOKENS)
        if has_only_keep_tokens:
            return False
        match_relative_regex = self._get_match_relative_regex_cache()
        for token in tokens:
            if token.isdigit() or match_relative_regex.match(token) or token in self:
                continue
            else:
                return False
        else:
            return True

    def split(self, string, keep_formatting=False):
        """
        Split the date string using translations in locale info.

        :param string:
            Date string to be splitted.
        :type string:
            str

        :param keep_formatting:
            If True, retain formatting of the date string.
        :type keep_formatting: bool

        :return: A list of string tokens formed after splitting the date string.
        """
        if not string:
            return string

        split_relative_regex = self._get_split_relative_regex_cache()
        match_relative_regex = self._get_match_relative_regex_cache()

        tokens = split_relative_regex.split(string)

        for i, token in enumerate(tokens):
            if match_relative_regex.match(token):
                tokens[i] = [token]
                continue
            tokens[i] = self._split_by_known_words(token, keep_formatting)

        return list(filter(bool, chain.from_iterable(tokens)))

    def _split_by_known_words(self, string, keep_formatting):
        if not string:
            return string

        regex = self._get_split_regex_cache()
        match = regex.match(string)
        if not match:
            return (self._split_by_numerals(string, keep_formatting)
                    if self._should_capture(string, keep_formatting) else [])

        unparsed, known, unknown = match.groups()
        splitted = [known] if self._should_capture(known, keep_formatting) else []
        if unparsed and self._should_capture(unparsed, keep_formatting):
            splitted = self._split_by_numerals(unparsed, keep_formatting) + splitted
        if unknown:
            splitted.extend(self._split_by_known_words(unknown, keep_formatting))

        return splitted

    def _split_by_numerals(self, string, keep_formatting):
        return [token for token in NUMERAL_PATTERN.split(string)
                if self._should_capture(token, keep_formatting)]

    def _should_capture(self, token, keep_formatting):
        return keep_formatting or token in ALWAYS_KEEP_TOKENS or KEEP_TOKEN_PATTERN.match(token)

    def _get_sorted_words_from_cache(self):
        if (
            self._settings.registry_key not in self._sorted_words_cache
            or self.info['name'] not in self._sorted_words_cache[self._settings.registry_key]
        ):
            self._sorted_words_cache.setdefault(
                self._settings.registry_key, {})[self.info['name']] = \
                sorted([key for key in self], key=len, reverse=True)
        return self._sorted_words_cache[self._settings.registry_key][self.info['name']]

    def _get_split_regex_cache(self):
        if (
            self._settings.registry_key not in self._split_regex_cache
            or self.info['name'] not in self._split_regex_cache[self._settings.registry_key]
        ):
            self._construct_split_regex()
        return self._split_regex_cache[self._settings.registry_key][self.info['name']]

    def _construct_split_regex(self):
        known_words_group = "|".join(map(re.escape, self._get_sorted_words_from_cache()))
        if self._no_word_spacing:
            regex = r"^(.*?)({})(.*)$".format(known_words_group)
        else:
            regex = r"^(.*?(?:\A|\W|_|\d))({})((?:\Z|\W|_|\d).*)$".format(known_words_group)
        self._split_regex_cache.setdefault(
            self._settings.registry_key, {})[self.info['name']] = \
            re.compile(regex, re.UNICODE | re.IGNORECASE)

    def _get_sorted_relative_strings_from_cache(self):
        if (
            self._settings.registry_key not in self._sorted_relative_strings_cache
            or self.info['name'] not in self._sorted_relative_strings_cache[self._settings.registry_key]
        ):
            self._sorted_relative_strings_cache.setdefault(
                self._settings.registry_key, {})[self.info['name']] = \
                sorted([PARENTHESES_PATTERN.sub('', key) for key in
                        self._relative_strings], key=len, reverse=True)
        return self._sorted_relative_strings_cache[self._settings.registry_key][self.info['name']]

    def _get_split_relative_regex_cache(self):
        if (
            self._settings.registry_key not in self._split_relative_regex_cache
            or self.info['name'] not in self._split_relative_regex_cache[self._settings.registry_key]
        ):
            self._construct_split_relative_regex()
        return self._split_relative_regex_cache[self._settings.registry_key][self.info['name']]

    def _construct_split_relative_regex(self):
        known_relative_strings_group = "|".join(self._get_sorted_relative_strings_from_cache())
        if self._no_word_spacing:
            regex = "({})".format(known_relative_strings_group)
        else:
            regex = "(?<=(?:\\A|\\W|_))({})(?=(?:\\Z|\\W|_))".format(known_relative_strings_group)
        self._split_relative_regex_cache.setdefault(
            self._settings.registry_key, {})[self.info['name']] = \
            re.compile(regex, re.UNICODE | re.IGNORECASE)

    def _get_match_relative_regex_cache(self):
        if (
            self._settings.registry_key not in self._match_relative_regex_cache
            or self.info['name'] not in self._match_relative_regex_cache[self._settings.registry_key]
        ):
            self._construct_match_relative_regex()
        return self._match_relative_regex_cache[self._settings.registry_key][self.info['name']]

    def _construct_match_relative_regex(self):
        known_relative_strings_group = "|".join(self._get_sorted_relative_strings_from_cache())
        regex = "^({})$".format(known_relative_strings_group)
        self._match_relative_regex_cache.setdefault(
            self._settings.registry_key, {})[self.info['name']] = \
            re.compile(regex, re.UNICODE | re.IGNORECASE)


class NormalizedDictionary(Dictionary):

    def __init__(self, locale_info, settings=None):
        super().__init__(locale_info, settings)
        self._normalize()

    def _normalize(self):
        new_dict = {}
        conflicting_keys = []
        for key, value in self._dictionary.items():
            normalized = normalize_unicode(key)
            if key != normalized and normalized in self._dictionary:
                conflicting_keys.append(key)
            else:
                new_dict[normalized] = value
        for key in conflicting_keys:
            normalized = normalize_unicode(key)
            if key in (self.info.get('skip', []) + self.info.get('pertain', [])):
                new_dict[normalized] = self._dictionary[key]
        self._dictionary = new_dict
        self._relative_strings = list(map(normalize_unicode, self._relative_strings))



info = {
    "name": "luo",
    "date_order": "DMY",
    "january": [
        "dac",
        "dwe mar achiel"
    ],
    "february": [
        "dar",
        "dwe mar ariyo"
    ],
    "march": [
        "dad",
        "dwe mar adek"
    ],
    "april": [
        "dan",
        "dwe mar ang'wen"
    ],
    "may": [
        "dah",
        "dwe mar abich"
    ],
    "june": [
        "dau",
        "dwe mar auchiel"
    ],
    "july": [
        "dao",
        "dwe mar abiriyo"
    ],
    "august": [
        "dab",
        "dwe mar aboro"
    ],
    "september": [
        "doc",
        "dwe mar ochiko"
    ],
    "october": [
        "dap",
        "dwe mar apar"
    ],
    "november": [
        "dgi",
        "dwe mar gi achiel"
    ],
    "december": [
        "dag",
        "dwe mar apar gi ariyo"
    ],
    "monday": [
        "wuok tich",
        "wut"
    ],
    "tuesday": [
        "tar",
        "tich ariyo"
    ],
    "wednesday": [
        "tad",
        "tich adek"
    ],
    "thursday": [
        "tan",
        "tich ang'wen"
    ],
    "friday": [
        "tab",
        "tich abich"
    ],
    "saturday": [
        "ngeso",
        "ngs"
    ],
    "sunday": [
        "jmp",
        "jumapil"
    ],
    "am": [
        "od"
    ],
    "pm": [
        "ot"
    ],
    "year": [
        "higa"
    ],
    "month": [
        "dwe"
    ],
    "week": [
        "juma"
    ],
    "day": [
        "chieng'"
    ],
    "hour": [
        "saa"
    ],
    "minute": [
        "dakika"
    ],
    "second": [
        "nyiriri mar saa"
    ],
    "relative-type": {
        "0 day ago": [
            "kawuono"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "nyoro"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "kiny"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "fur",
    "date_order": "DMY",
    "january": [
        "zen",
        "zenâr"
    ],
    "february": [
        "fev",
        "fevrâr"
    ],
    "march": [
        "mar",
        "març"
    ],
    "april": [
        "avr",
        "avrîl"
    ],
    "may": [
        "mai"
    ],
    "june": [
        "jug",
        "jugn"
    ],
    "july": [
        "lui"
    ],
    "august": [
        "avo",
        "avost"
    ],
    "september": [
        "set",
        "setembar"
    ],
    "october": [
        "otu",
        "otubar"
    ],
    "november": [
        "nov",
        "novembar"
    ],
    "december": [
        "dic",
        "dicembar"
    ],
    "monday": [
        "lun",
        "lunis"
    ],
    "tuesday": [
        "mar",
        "martars"
    ],
    "wednesday": [
        "mie",
        "miercus"
    ],
    "thursday": [
        "joi",
        "joibe"
    ],
    "friday": [
        "vin",
        "vinars"
    ],
    "saturday": [
        "sab",
        "sabide"
    ],
    "sunday": [
        "dom",
        "domenie"
    ],
    "am": [
        "a"
    ],
    "pm": [
        "p"
    ],
    "year": [
        "an"
    ],
    "month": [
        "mês"
    ],
    "week": [
        "setemane"
    ],
    "day": [
        "dì"
    ],
    "hour": [
        "ore"
    ],
    "minute": [
        "minût"
    ],
    "second": [
        "secont"
    ],
    "relative-type": {
        "0 day ago": [
            "vuê"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "îr"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "doman"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) zornade indaûr",
            "(\\d+[.,]?\\d*) zornadis indaûr"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) ore indaûr",
            "(\\d+[.,]?\\d*) oris indaûr"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) minût indaûr",
            "(\\d+[.,]?\\d*) minûts indaûr"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) mês indaûr"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) secont indaûr",
            "(\\d+[.,]?\\d*) seconts indaûr"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) setemane indaûr",
            "(\\d+[.,]?\\d*) setemanis indaûr"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) agns indaûr",
            "(\\d+[.,]?\\d*) an indaûr"
        ],
        "in \\1 day": [
            "ca di (\\d+[.,]?\\d*) zornade",
            "ca di (\\d+[.,]?\\d*) zornadis"
        ],
        "in \\1 hour": [
            "ca di (\\d+[.,]?\\d*) ore",
            "ca di (\\d+[.,]?\\d*) oris"
        ],
        "in \\1 minute": [
            "ca di (\\d+[.,]?\\d*) minût",
            "ca di (\\d+[.,]?\\d*) minûts"
        ],
        "in \\1 month": [
            "ca di (\\d+[.,]?\\d*) mês"
        ],
        "in \\1 second": [
            "ca di (\\d+[.,]?\\d*) secont",
            "ca di (\\d+[.,]?\\d*) seconts"
        ],
        "in \\1 week": [
            "ca di (\\d+[.,]?\\d*) setemane",
            "ca di (\\d+[.,]?\\d*) setemanis"
        ],
        "in \\1 year": [
            "ca di (\\d+[.,]?\\d*) agns",
            "ca di (\\d+[.,]?\\d*) an"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



from dateparser.search.search import DateSearchWithDetection


_search_with_detection = DateSearchWithDetection()


def search_dates(text, languages=None, settings=None, add_detected_language=False, detect_languages_function=None):
    """Find all substrings of the given string which represent date and/or time and parse them.

        :param text:
            A string in a natural language which may contain date and/or time expressions.
        :type text: str

        :param languages:
            A list of two letters language codes.e.g. ['en', 'es']. If languages are given, it will
            not attempt to detect the language.
        :type languages: list

        :param settings:
            Configure customized behavior using settings defined in :mod:`dateparser.conf.Settings`.
        :type settings: dict

        :param add_detected_language:
            Indicates if we want the detected language returned in the tuple.
        :type add_detected_language: bool

        :param detect_languages_function:
            A function for language detection that takes as input a `text` and a `confidence_threshold`,
            and returns a list of detected language codes.
            Note: detect_languages_function is only uses if `languages` are not provided.
        :type detect_languages_function: function

        :return: Returns list of tuples containing:
            substrings representing date and/or time, corresponding :mod:`datetime.datetime`
            object and detected language if *add_detected_language* is True.
            Returns None if no dates that can be parsed are found.
        :rtype: list
        :raises: ValueError - Unknown Language

        >>> from dateparser.search import search_dates
        >>> search_dates('The first artificial Earth satellite was launched on 4 October 1957.')
        [('on 4 October 1957', datetime.datetime(1957, 10, 4, 0, 0))]

        >>> search_dates('The first artificial Earth satellite was launched on 4 October 1957.',
        >>>              add_detected_language=True)
        [('on 4 October 1957', datetime.datetime(1957, 10, 4, 0, 0), 'en')]

        >>> search_dates("The client arrived to the office for the first time in March 3rd, 2004 "
        >>>              "and got serviced, after a couple of months, on May 6th 2004, the customer "
        >>>              "returned indicating a defect on the part")
        [('in March 3rd, 2004 and', datetime.datetime(2004, 3, 3, 0, 0)),
         ('on May 6th 2004', datetime.datetime(2004, 5, 6, 0, 0))]

        """
    result = _search_with_detection.search_dates(
        text=text, languages=languages, settings=settings, detect_languages_function=detect_languages_function
    )
    dates = result.get('Dates')
    if dates:
        if add_detected_language:
            language = result.get('Language')
            dates = [date + (language, ) for date in dates]
        return dates



info = {
    "name": "ebu",
    "date_order": "DMY",
    "january": [
        "mbe",
        "mweri wa mbere"
    ],
    "february": [
        "kai",
        "mweri wa kaĩri"
    ],
    "march": [
        "kat",
        "mweri wa kathatũ"
    ],
    "april": [
        "kan",
        "mweri wa kana"
    ],
    "may": [
        "gat",
        "mweri wa gatano"
    ],
    "june": [
        "gan",
        "mweri wa gatantatũ"
    ],
    "july": [
        "mug",
        "mweri wa mũgwanja"
    ],
    "august": [
        "knn",
        "mweri wa kanana"
    ],
    "september": [
        "ken",
        "mweri wa kenda"
    ],
    "october": [
        "iku",
        "mweri wa ikũmi"
    ],
    "november": [
        "imw",
        "mweri wa ikũmi na ũmwe"
    ],
    "december": [
        "igi",
        "mweri wa ikũmi na kaĩrĩ"
    ],
    "monday": [
        "njumatatu",
        "tat"
    ],
    "tuesday": [
        "ine",
        "njumaine"
    ],
    "wednesday": [
        "njumatano",
        "tan"
    ],
    "thursday": [
        "aramithi",
        "arm"
    ],
    "friday": [
        "maa",
        "njumaa"
    ],
    "saturday": [
        "njumamothii",
        "nmm"
    ],
    "sunday": [
        "kiumia",
        "kma"
    ],
    "am": [
        "ki"
    ],
    "pm": [
        "ut"
    ],
    "year": [
        "mwaka"
    ],
    "month": [
        "mweri"
    ],
    "week": [
        "kiumia"
    ],
    "day": [
        "mũthenya"
    ],
    "hour": [
        "ithaa"
    ],
    "minute": [
        "ndagĩka"
    ],
    "second": [
        "sekondi"
    ],
    "relative-type": {
        "0 day ago": [
            "ũmũnthĩ"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "ĩgoro"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "rũciũ"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "ga",
    "date_order": "DMY",
    "january": [
        "ean",
        "eanáir"
    ],
    "february": [
        "feabh",
        "feabhra"
    ],
    "march": [
        "márta"
    ],
    "april": [
        "aib",
        "aibreán"
    ],
    "may": [
        "beal",
        "bealtaine"
    ],
    "june": [
        "meith",
        "meitheamh"
    ],
    "july": [
        "iúil"
    ],
    "august": [
        "lún",
        "lúnasa"
    ],
    "september": [
        "meán fómhair",
        "mfómh"
    ],
    "october": [
        "deireadh fómhair",
        "dfómh"
    ],
    "november": [
        "samh",
        "samhain"
    ],
    "december": [
        "noll",
        "nollaig"
    ],
    "monday": [
        "dé luain",
        "luan"
    ],
    "tuesday": [
        "dé máirt",
        "máirt"
    ],
    "wednesday": [
        "céad",
        "dé céadaoin"
    ],
    "thursday": [
        "déar",
        "déardaoin"
    ],
    "friday": [
        "aoine",
        "dé haoine"
    ],
    "saturday": [
        "dé sathairn",
        "sath"
    ],
    "sunday": [
        "domh",
        "dé domhnaigh"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "bl",
        "bliain"
    ],
    "month": [
        "mí"
    ],
    "week": [
        "scht",
        "seachtain"
    ],
    "day": [
        "lá"
    ],
    "hour": [
        "u",
        "uair"
    ],
    "minute": [
        "n",
        "nóim",
        "nóiméad"
    ],
    "second": [
        "s",
        "soic",
        "soicind"
    ],
    "relative-type": {
        "0 day ago": [
            "inniu"
        ],
        "0 hour ago": [
            "an uair seo"
        ],
        "0 minute ago": [
            "an nóiméad seo"
        ],
        "0 month ago": [
            "an mhí seo"
        ],
        "0 second ago": [
            "anois"
        ],
        "0 week ago": [
            "an tscht seo",
            "an tseachtain seo"
        ],
        "0 year ago": [
            "an bhl seo",
            "an bhliain seo"
        ],
        "1 day ago": [
            "inné"
        ],
        "1 month ago": [
            "an mhí seo caite"
        ],
        "1 week ago": [
            "an tscht seo caite",
            "an tseachtain seo caite"
        ],
        "1 year ago": [
            "anuraidh"
        ],
        "in 1 day": [
            "amárach"
        ],
        "in 1 month": [
            "an mhí seo chugainn"
        ],
        "in 1 week": [
            "an tscht seo chugainn",
            "an tseachtain seo chugainn"
        ],
        "in 1 year": [
            "an bhl seo chugainn",
            "an bhliain seo chugainn"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) lá ó shin"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) uair an chloig ó shin",
            "(\\d+[.,]?\\d*) uair ó shin"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) nóim ó shin",
            "(\\d+[.,]?\\d*) nóiméad ó shin"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) mhí ó shin",
            "(\\d+[.,]?\\d*) mí ó shin"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) soic ó shin",
            "(\\d+[.,]?\\d*) soicind ó shin"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) scht ó shin",
            "(\\d+[.,]?\\d*) seachtain ó shin"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) bhl ó shin",
            "(\\d+[.,]?\\d*) bhliain ó shin",
            "(\\d+[.,]?\\d*) bl ó shin",
            "(\\d+[.,]?\\d*) bliain ó shin"
        ],
        "in \\1 day": [
            "i gceann (\\d+[.,]?\\d*) lá"
        ],
        "in \\1 hour": [
            "i gceann (\\d+[.,]?\\d*) uair",
            "i gceann (\\d+[.,]?\\d*) uair an chloig"
        ],
        "in \\1 minute": [
            "i gceann (\\d+[.,]?\\d*) nóim",
            "i gceann (\\d+[.,]?\\d*) nóiméad"
        ],
        "in \\1 month": [
            "i gceann (\\d+[.,]?\\d*) mhí",
            "i gceann (\\d+[.,]?\\d*) mí"
        ],
        "in \\1 second": [
            "i gceann (\\d+[.,]?\\d*) soic",
            "i gceann (\\d+[.,]?\\d*) soicind"
        ],
        "in \\1 week": [
            "i gceann (\\d+[.,]?\\d*) scht",
            "i gceann (\\d+[.,]?\\d*) seachtain"
        ],
        "in \\1 year": [
            "i gceann (\\d+[.,]?\\d*) bhliain",
            "i gceann (\\d+[.,]?\\d*) bl",
            "i gceann (\\d+[.,]?\\d*) bliain"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}
